import numpy as np
from numpy import ma as ma
from netCDF4 import Dataset as nc
import sys
from scipy.spatial import ckdtree
from glob import glob
import shutil
import gzip

def getvarmask(varin):
    """define if the name of the mask corresponding to each variable
    input:
    varin: name of the input variables
    output:
    mask: name of the corresponding mask variables
    varlon: name of the longitude variable
    varlat: name of the latitude variables"""

    if varin in ['avmu','un','ub','gru','gsu','gtu','ssu_m','u_io']:
        mask='umask' ; varlon='glamu' ; varlat='gphiu' 
    elif varin in ['avmv','vn','vb','grv','gsv','gtv','ssv_m','v_io']:
        mask='vmask' ; varlon='glamv' ; varlat='gphiv'  
    elif varin in ['rotb','rotn']:
        mask='fmask' ; varlon='glamf' ; varlat='gphif' 
    elif varin in ['avt','avmt','en','dissl','hdivb','hdivn','rhop','so','thetao','sb','tb','gcx','gcxb','gcxbfs','ssh_m','sshb','sshn','sss_m','sst_m','alb_ice','sst_io','sss_io']:
        mask='tmask' ; varlon='glamt' ; varlat='gphit' 
    else:
        print("variable not found")
        sys.exit(1)
    return(mask, varlon, varlat)
    
def read_files(fin, varin, fmin, fmout):
    """This function is reading the variables needed for the interpolation
    process from the input files
    Arguments:
    filein: Input restart type netcdf (already open)
    varin: Variable to interpolate
    varmask: name of the mask variable corresponding to varin 
            (tmask, umask, vmask, fmask)
    fmin: Input maskfile type netcdf (already open) 
    fmout: Output mask file type netcdf (already open)
    Outputs:
    fieldin: Field to interpolate
    depthin: 3D depth corresponding to fieldin
    thickin: 3D thichness corresponding to the fieldin
    thickout: 3D thickness toward which the interpolation 
              has to be performed
    depthout: 3D depth toward which the interpolation 
              has to be performed
    """
    
    #get the mask variable corresponding to varin
    varmask, varlon, varlat =getvarmask(varin)

    
    #read the input mask (needed because original s4 not masked)
    maskin=np.array(fmin.variables[varmask][:].squeeze())
    #generate a masked array to mask the land
    maskin=ma.array(maskin>=1,mask=maskin<0.5)
    #read the thickness
    thickin=np.array(fmin.variables["e3t_0"][:].squeeze())
    #read the depth
    depthin=np.array(fmin.variables["gdept_0"][:].squeeze())
    #read the longitude
    lonin=np.array(fmin.variables[varlon][:].squeeze())
    #read the latitude
    latin=np.array(fmin.variables[varlat][:].squeeze())

    
    fieldin=np.array(fin.variables[varin][:].squeeze())
    #diemnsion name
    dimnames=list(fin.variables[varin].dimensions)

    

    #if the last level of the input field is fully masked ()
    #or almost fully masked (less than 20points)
    #remove it
    #check if the variable is 2D or 3D:
    if len(fieldin.shape)==3:
        lastlev=maskin.shape[0]-1
        if (np.sum(maskin[lastlev,:,:])<=20) or (ma.sum(maskin[lastlev,:,:]).mask):
            maskin=maskin[0:lastlev,:,:]
            fieldin=np.array(fieldin[0:lastlev,:,:])
            thickin=thickin[0:lastlev]
            depthin=depthin[0:lastlev]


    #read the mask field
    maskout=np.array(fmout.variables[varmask][:].squeeze())
    #thickness
    thickout=np.array(fmout.variables["e3t_0"][:].squeeze())
    #depth
    depthout=np.array(fmout.variables["gdept_0"][:].squeeze())
    #read the longitude
    lonout=np.array(fmout.variables[varlon][:].squeeze())
    #read the latitude
    latout=np.array(fmout.variables[varlat][:].squeeze())
    #generate a masked array to mask the land
    maskout=ma.array(maskout,mask=maskout<0.5)

    #permute the dimensions if needed
    # not take into account t dimension (size 1)
    if len(fieldin.shape)==3:
        correctdimorder=(u'z', u'y', u'x')
    else:
        correctdimorder=(u'y', u'x')

    dimnames.remove(u't')
    #the correctdimension order
    for idim,dimname in enumerate(dimnames):
        if dimname!=correctdimorder[idim]:
            print("Dimension"+dimname+"permuted")
            fieldin=moveaxis(fieldin, idim, 
                         correctdimorder.index(dimname))
    
    # extend the input depth in 3D 
    #(needed because from Nemo 3.6 both vary with time ans space)
    lz0, ly0, lx0 = maskin.shape
    lzout, lyout, lxout = maskout.shape
    if len(depthin.shape)==1:
        depthin.shape=(lz0,1,1)
        depthin=np.repeat(np.repeat(depthin,ly0, axis=1), lx0, axis=2)
    if len(depthout.shape)==1:
        depthout.shape=(lzout,1,1)
        depthout=np.repeat(np.repeat(depthout, ly0, axis=1), lx0, axis=2)
    # extend the input thickness in 3D
    if len(thickin.shape)==1:
        thickin.shape=(lz0,1,1)
        thickin=np.repeat(np.repeat(thickin,ly0, axis=1), lx0, axis=2)
    if len(thickout.shape)==1:
        thickout.shape=(lzout,1,1)
        thickout=np.repeat(np.repeat(thickout,ly0, axis=1), lx0, axis=2)


    #construct mask and thickness for 2D variables
    if len(fieldin.shape)==2:
        ly0,lx0=fieldin.shape
        maskin=maskin[0,:,:]
        thickin=np.zeros((ly0,lx0),'d')
        depthin=np.zeros((ly0,lx0),'d')
        thickout=np.zeros((ly0,lx0),'d')
        depthout=np.zeros((ly0,lx0),'d')
        maskout=maskout[0,:,:]

    #check that the mask corresponds to the input file
    #(in s4 restarts continents are filled with 0)
    
    if (fieldin[np.where(maskin.mask)]!=0).any():
        print("the variable: "+varmask)
        print("the input mask:")
        print(fmin.filepath())
        print("does not corespond to the input file:")
        print(fin.filepath())
        print("for the variable:"+varin)
        sys.exit(1)

    #apply the mask
    fieldin=ma.array(fieldin,mask=1-maskin)

    #check that the field is not empty
    if  (np.sum(fieldin)==0):
        print("The field written from the file: "+f)
        print("for the variable"+varin)
        print("mask with the file: "+fmask)
        print("seems to be empty")
        print("Please check your input")
        sys.exit(1)
    
    return(fieldin, thickin, depthin, lonin, latin, 
           maskout, thickout, depthout, lonout, latout)

def interp_vert(fieldin, thick_in, thick_out):
    """This function interpolates vertically a (x,y,z) field using the
    conservative method.
    Arguments:
    fieldin: Input Field type mask array
    thick_int: input thickness thickness of the level 
    thick_out: output thickness of the level 
    History : Chloe Prodhomme - Initial version adpated
              from Cfutools extrap_vert function
              written by V. Guemas - 2016"""

    (lz0,ly0,lx0)=fieldin.shape
    (lz1,ly1,lx1)=thick_out.shape

    var0_a=np.array(fieldin)
    var0=np.array(fieldin)
    var0=ma.masked_array(var0,mask=None,fill_value=None)

    var0=var0.astype('d')

    mask_in=1-fieldin.mask
    
    var1=np.zeros((lz1,ly0,lx0),'d')
    thick_jk1=np.zeros((ly0,lx0),'d')
    mask_out=np.zeros((lz1,ly0,lx0),'d')
    thick_bis=np.zeros((ly0,lx0),'d')

    jk0=0
    for jk1 in np.arange(lz1) :
        while ( (jk0 <= (lz0-1)) & ((np.add.reduce(thick_in[0:(jk0+1)]) <= np.add.reduce(thick_out[0:(jk1+1)])) ).any()) :
            thick_bis=np.multiply(np.minimum(thick_in[jk0],np.add.reduce(thick_in[0:(jk0+1)])-np.add.reduce(thick_out[0:jk1])),mask_in[jk0,:,:])
            var1[jk1,:,:]=var1[jk1,:,:]+np.multiply(var0[jk0,:,:],thick_bis)
            thick_jk1=thick_jk1+thick_bis
            jk0=jk0+1
        if True: #( jk0 <= (lz0-1)) :
            jk0in=min(jk0,lz0-1)
            thick_bis=np.multiply(np.minimum(thick_out[jk1],np.add.reduce(thick_out[0:(jk1+1)])-np.add.reduce(thick_in[0:jk0])),mask_in[jk0in,:,:])
            var1[jk1,:,:]=var1[jk1,:,:]+np.multiply(var0[jk0in,:,:],thick_bis)
            thick_jk1=thick_jk1+thick_bis
        mask_out[jk1,:,:]=np.where(thick_jk1>0.01,False,True)
        thick_jk1=np.where(thick_jk1==0.,1.,thick_jk1)
        var1[jk1,:,:]=var1[jk1,:,:]/thick_jk1
        thick_jk1[:,:]=0
    #mask the output field
    fieldout=ma.array(var1, mask=mask_out)
    
    return(fieldout)


def lon_lat_to_cartesian(lon, lat, R=1):
    """calculates lon, lat coordinates of a point on a sphere with
    radius R
    """
    lon_r = np.radians(lon)
    lat_r = np.radians(lat)
    x = R * np.cos(lat_r) * np.cos(lon_r)
    y = R * np.cos(lat_r) * np.sin(lon_r)
    z = R * np.sin(lat_r)
    return x, y, z


def extrap(fieldin,lonin,latin,maskout):
    """This function extrapolates horizontally each level of a (x,y,z)
    field using the nearest neighbour method.
    Argument:
    fieldin: Input field (ma.array type)
    lonin: 2d longitude
    latin: 2d longitude
    maskout: mask of the field after extrapolation
    Output:
    fieldout: output field (ma.array)
    """
    #add a dimension of size 1 for 2d field
    if len(fieldin.shape)==2:
        fieldin.shape=(1,)+fieldin.shape

    #construct points to fill
    Pfill=ma.array((fieldin.mask)&(maskout!=0), mask=1-maskout)

    # fieldout intitialization
    fieldout=np.array(fieldin)
    
    
    #each level has a different mask, loop over the levels
    lz0=fieldin.shape[0]
        
    for lev in np.arange(lz0):
        newvar = ma.array(fieldin[lev,:,:], 
                          mask=fieldin.mask[lev,:,:])
        Pfilllev = Pfill[lev,:,:] 
        
        #not masked coordinates for the present level
        nomlon = lonin[np.where(newvar.mask==False)]
        nomlat = latin[np.where(newvar.mask==False)]
        #cartesian coordinates of not masked points
        levxs, levys, levzs = lon_lat_to_cartesian(nomlon, nomlat)
        levkdt = ckdtree.cKDTree(np.array([levxs, levys, levzs]).T)
        #points to extrapolate at the current level
        newindexes = np.where(Pfilllev==1)
        latidxs = newindexes[0]
        lonidxs = newindexes[1]
        #print(newindexes)
        exlon = lonin[latidxs, lonidxs]
        exlat = latin[latidxs, lonidxs]
        #print(newvar[levkdt.data[0,0], levkdt.data[0,1]])
               
        #cartesian coordinates of the points to extrapolate
        xt, yt, zt = lon_lat_to_cartesian(exlon, exlat)
        #calculates distances and indexes of the nearest neighbours
        dist, idx = levkdt.query(np.array([xt, yt, zt]).T, k=1, p=2)

        #assign to the result matrix the found values
        fieldout[lev,latidxs, lonidxs] = newvar[np.where(newvar.mask==False)].flatten()[idx]

    #mask field out
    fieldout=fieldout*maskout
    fieldout=ma.array(fieldout,mask=1-maskout)
    return(fieldout)



def vertinterp_extrap(filein, varin, fmask, fmaskout, month="", modvers=""):
    """read and interpolate verticaly and extrapolate horizontal the given variable
    from a restart file
    Arguments:
    filein: Input restart file
    varin: name of the variable to extrapolate
    fmask: mask file corresponding to varin
    fmaskout: mask to define the grid toward which the extrapolation if done
    month: starting month of the restart
           (needed only if there are points to fill in the caspienne sea)
    modelvers: name of the model version to find the climatology to fill the caspienne
          (needed only if there are points to fill in the caspienne sea)  
   
    """
    
    #read files and define variables needed for the rest of the process
    (fieldin, thickin, depthin, lonin, latin, 
     maskout, 
     thickout, depthout, 
     lonout, latout) = read_files(filein, 
                                varin, fmask, fmaskout)

    #check if the variable is 2D or 3D:
    if len(fieldin.shape)==3:
        #perform the vertical extrapolation
        fieldin=interp_vert(fieldin, thickin,thickout)
    
    #construct points to fill
    Pfill=ma.array((fieldin.mask)&(maskout!=0), mask=1-maskout)
    
    #find if the caspienne sea need to be filled
    caspbox=np.where((lonin>45)&(lonin<56)&(latin>35)&(latin<50))
    #longitude and latitude boundary of the caspienne sea
    loncasp1,loncasp2=caspbox[0].min(),caspbox[0].max()
    latcasp1,latcasp2=caspbox[1].min(),caspbox[1].max()
    #check is there are points to fill in the caspienne sea
    if len(fieldin.shape)==2:
        casptest=np.sum(Pfill[loncasp1:loncasp2,latcasp1:latcasp2], 
                            axis=(0,1))!=0
    else: 
        casptest=np.sum(Pfill[:,loncasp1:loncasp2,latcasp1:latcasp2], 
                            axis=(0,1,2))!=0
    #If there are points to be filled
    if  casptest:

        #if ((varin=="so")|(varin=="so"))|((varin=="thetao")|(varin=="thetao")):
        if ((varin=="so")|(varin=="thetao")):

        #Ugly solution, to be improve would need levitus file at all resolution
            if modvers=="Ec3.1":
                pathclim="/esnas/releases/ic/ocean/ORCA1L46/levitus_clim/"
                if ((varin=="tn")|(varin=="tb")):
                    #name of the file
                    varclim="votemper"
                    fileclim=pathclim+"temperature_m%02d.nc"%month
            
                if ((varin=="sn")|(varin=="sb")):
                    #open climatological files
                    varclim="vosaline"
                    fileclim=pathclim+"salinity_m%02d.nc"%month  

            elif modvers=="Ec2.3":
                #in this case we use an old restart for the climatology
                fileclim=glob("/esnas/releases/ic/ocean/ORCA1/s4/s4_fc0_1993%02d*_restart.nc.gz"%month)
                print(fileclim)
                shutil.copy(fileclim[0], "fileclim.nc.gz")
                fileclim="fileclim.nc"
                inF = gzip.open('fileclim.nc.gz', 'rb')
                outF = open(fileclim, 'wb')
                outF.write( inF.read() )
                inF.close()
                outF.close()
                varclim=varin
            else:
                print("no climatology is available for this model version to fill the caspienne sea:")
                print(modvers)
                sys.exit(1)

            #open the file
            print(fileclim)
            fclim=nc(fileclim, "r")
            #read variable for the given month
            clim=np.array(fclim.variables[varclim][:].squeeze())

            #add the caspienne sea
            fieldin[:,caspbox[0],caspbox[1]]=clim[:,caspbox[0],caspbox[1]]
        else:
            print(fieldin.shape)
            print(caspbox[1].min())
            print(caspbox[1].max())
            print(caspbox[0].min())
            print(caspbox[0].max())
            if (len(fieldin.shape)==3):
                fieldin[:,caspbox[0],caspbox[1]]=0
            elif  (len(fieldin.shape)==2):
                fieldin[caspbox[0],caspbox[1]]=0
            else:
                print("vertinterp_extrap function can handle only 2 and 3d arrays")
                sys.exit(1)
    
    #horizontal extrapolation
    fieldout=extrap(fieldin,lonin,latin,maskout)
    return(fieldout)


    
    
