import os
from six import iteritems, string_types
import numpy as np
import iris
from iris.cube import CubeList
from iris.coords import AuxCoord
from iris.util import squeeze
import yaml
from numba import vectorize, int8, float32

def create_mask(mesh_file, maskglo_file, config_file=None):
    if not config_file:
        cwd = os.path.dirname(os.path.realpath(__file__))
        config_file = os.path.join(cwd, 'basins.yml')

    with open(config_file, 'r') as yfile:
        config = yaml.safe_load(yfile)

    sea_mask = squeeze(iris.load_cube(mesh_file, 'tmask'))[0, ...]
    lat = iris.load_cube(mesh_file, 'nav_lat').data
    lon = iris.load_cube(mesh_file, 'nav_lon').data
    basins = CubeList()
    basin_model = sea_mask.copy()

    basins_from_file = {
        'Atlantic_Ocean': 'tmaskatl',
        'Pacific_Ocean': 'tmaskpac',
        'Indian_Ocean': 'tmaskind',
    }
    for basin, variable in iteritems(basins_from_file):
        mask = squeeze(iris.load_cube(maskglo_file, variable)).data.astype(np.int8)
        basin_mask = basin_model.copy() * mask
        basins.append(prepare_cube(basin_mask, basin))

    print('\nCreating basins from boxes')
    for basin, basin_config in iteritems(config['basins']):
        print('Creating basin', basin, 'from', basin_config)
        mask =  create_basin(flatten_list(basin_config), lat, lon, basins)
        basin_mask = basin_model.copy() * mask
        basins.append(prepare_cube(basin_mask, basin))

    print('Merging and saving')
    basins = basins.merge_cube()
    basins.add_aux_coord(
        AuxCoord(lat, 'latitude', 'nav_lat'),
        (1, 2)
    )
    basins.add_aux_coord(
        AuxCoord(lat, 'longitude', 'nav_lon'),
        (1, 2)
    )
    iris.save(
        basins,
        'basins.nc',
        zlib=True,
        chunksizes=(1, basins.shape[1], basins.shape[2])
    )

    import matplotlib
    matplotlib.use('Qt5Agg')
    import matplotlib.pyplot as plt
    import iris.quickplot as qplt
    print(basins)
    basins.remove_coord('latitude')
    basins.remove_coord('longitude')
    for basin_slice in basins.slices_over('region'):
        basin_slice.long_name = basin_slice.coord('region').points[0]
        basin_slice.standard_name = None
        qplt.pcolormesh(basin_slice)
        plt.show()

def flatten_list(config):
    final_list = []
    for conf in config:
        if isinstance(conf, dict) or isinstance(conf, string_types):
            final_list.append(conf)
        else:
            final_list.extend(conf)
    return final_list

def create_basin(box_list, lat, lon, basins):
    if not box_list:
        return None

    mask = None
    for box in box_list:
        if 'lat' in box:
            current_box = apply_config(box['lat'][0], box['lat'][1], lat) * apply_config(box['lon'][0], box['lon'][1], lon)
            if mask is None:
                mask = current_box
                continue
            next_mask = current_box
        elif 'basin' in box:
            sea = basins.extract(iris.Constraint(region=box['basin']))[0]
            if mask is None:
                mask = sea.data.copy()
                continue
            next_mask = sea.data
        action = box.get('action', 'add')
        if action == 'add':
            mask = add(mask, next_mask)
        elif action == 'remove':
            mask = remove(mask, next_mask)
        elif action == 'intersection':
            mask = intersection(mask, next_mask)

    return mask

def prepare_cube(cube, basin):
    cube.add_aux_coord(AuxCoord([basin], var_name='region'))
    cube.var_name = 'basin'
    cube.standard_name = 'region'
    cube.long_name = 'Regional masks'
    cube.attributes = {}
    return cube

@vectorize([int8(float32, float32, float32)], nopython=True)
def apply_config(min, max, data):
    if data < min:
        return 0
    if data > max:
        return 0
    return 1

@vectorize([int8(int8, int8)], nopython=True)
def add(mask1, mask2):
    if mask1 + mask2 > 0:
        return 1
    return 0

@vectorize([int8(int8, int8)], nopython=True)
def remove(mask1, mask2):
    if mask1 - mask2 < 1:
        return 0
    return 1

@vectorize([int8(int8, int8)], nopython=True)
def intersection(mask1, mask2):
    if mask1 and mask2:
        return 1
    return 0


if __name__ == "__main__":
    create_mask(
        '/Users/nube/cpg_tools/new_masks/mesh_mask_nemo.Ec3.2_O1L75.nc',
        '/Users/nube/cpg_tools/new_masks/new_maskglo.Ec3.2_O1L75.nc',
    )