# coding=utf8
# Author  - Francois Massonnet
# Date    - 2 September 2016
#
# Purpose - Fills an oceanic region of a given land-sea mask
#
# Details - A land-sea mask consisting of values 1 (sea) and 0 (land)
#           is provided as input. Then an initial point, defined by
#           (lon, lat) is given by the user. From there, the script iteratively
#           explores all neighbours of that point until it hits a coastline.
#
# Output produced - A NetCDF file with values "1" in the basin that was filled.

import matplotlib.pyplot as plt
import numpy as np
import os
import shutil
from netCDF4 import Dataset

# ----------------
# INPUT PARAMETERS
# ----------------

# The name of the input mask
maskin = "/esnas/autosubmit/con_files/mesh_mask_nemo.Ec3.2_O1L75.nc"

# The name of the variable defining the mask in that file,
# and the names of lat and lon variables (at T-point).
maskvar = "tmaskutil"
latvar = "gphit"
lonvar = "glamt"
create_gif = True

# The list of initial points that have to be marked as "2"
# The Python convention is to be followed. If the point in the 
# NetCDF file is (i = 40, j= 30) (for example), then you just
# have to swap the two: ilist = [30] and jlist = [40] (NetCDF
# calls "i" the left-right direction while in this script "i"
# runs along the columns.

# Atlantic + Arctic
# ilist = [255, 218, 230, 199, 237, 166, 111]
# jlist = [110, 256, 297, 313, 236, 257, 275]

# Pacific
ilist = [118, 206, 141, 151, 183]
jlist = [85, 139, 183, 130, 63]

# Indian
# ilist = [121, 183, 138, 145]
# jlist = [11,  326, 351, 46]

# The list of points that define barriers. For example, if you want
# to fill the Mediterranean Sea, you should close Gibraltar.
# The indexing follows Python's conventions here: i_closed is the row,
# j_closed the column


# Atlantic mask
# j_closed = [113, 114, 115, 113]      # Bering strait
# i_closed = [244, 244, 244, 245]      # Bering strait
# j_closed.extend(range(233,310)) # Southern limit
# i_closed.extend([94] * (310 - 233)) # Southern limit

# Pacific mask
j_closed = range(70, 223)  # Southern limit
i_closed = [89] * (223 - 70)  # Southern limit
j_closed.extend([113, 114, 115])  # Bering strait
i_closed.extend([245, 245, 245])  # Bering strait
i_closed.extend([121])  # Torres strait
j_closed.extend([70])  # Torres strait
i_closed.extend([141, 141])  # Sea of Moluques
j_closed.extend([54, 57])  # Sea of Moluques
i_closed.extend([149])  # Strait between Borneo and Sulawesi
j_closed.extend([47])  # Strait between Borneo and Sulawesi
i_closed.extend([162])  # Strait between Philippines and Borneo
j_closed.extend([47])  # Strait between Philippines and Borneo
i_closed.extend([181, 181, 182, 186, 187, 188, 189, 190, 191])
j_closed.extend([49, 48, 48, 48, 48, 48, 48, 48, 48])

# Indian mask
# j_closed = range(0, 180) # Southern limit, Australia side
# i_closed = [93] * 180 # Southern limit, Australia side

# j_closed.extend(range(181, nj))  # Southern limit, Africa side
# i_closed.extend([94] * (nj - 181))

# j_closed.extend([49,  49,  49,  49,  49,  49,  49,  49,  49,  47,  47,  54,  57,  70])  # Indonesian basin
# i_closed.extend([189, 188, 187, 186, 185, 184, 183, 182, 181, 161, 150, 142, 142, 121]) # Indonesian basin


# The name of the output file and the variable name of the output
outfile = 'tmaskpac.nc'
varoutname = 'tmaskpac'

# -----------------------
# END OF INPUT PARAMETERS
# -----------------------

# Step 0: Opening the grid
# ------------------------
f = Dataset(maskin, mode='r')
inimask = np.squeeze(f.variables[maskvar][:])
lat = np.squeeze(f.variables[latvar][:])
lon = np.squeeze(f.variables[lonvar][:])
f.close()

# Get the dimensions
ni = inimask.shape[0]
nj = inimask.shape[1]

JJ, II = np.meshgrid(np.arange(nj), np.arange(ni))

# Check that initial points are not land points
for initial_point in np.arange(len(ilist)):
    if inimask[ilist[initial_point], jlist[initial_point]] == 0:
        print("Initial point ({0}, {1}) is on land - nothing will arise from there".format(ilist[initial_point],
                                                                                           jlist[initial_point]))
    else:
        # Mark initial points as '2'
        inimask[ilist[initial_point], jlist[initial_point]] = 2

# Adding the barrier points provided by the user
for barrier_point in np.arange(len(i_closed)):
    inimask[i_closed[barrier_point], j_closed[barrier_point]] = 0

if os.path.exists('figs'):
    shutil.rmtree('figs')

os.mkdir('figs')

plt.figure()
plt.pcolor(JJ, II, inimask)
plt.savefig("figs/iteration0000.png")
plt.close("all")

# Then iterate!
n = len(ilist)  # length of the list
nm = np.sum(1 * (inimask == 2))  # number of points already marked '2'
k = 1  # for figure drawing
previous_nm = 0.0

while previous_nm < nm:
    previous_nm = nm
    for jn in np.arange(n):

        # 1. Look for the point immediately to the right
        # Only do something if the neighbour is within the domain
        ii = ilist[jn]
        jj = jlist[jn] + 1

        if jj < nj:
            # If the point is ocean, set it to '2'
            if inimask[ii, jj] == 1:
                inimask[ii, jj] = 2
                # Add the point to the list
                ilist.append(ii)
                jlist.append(jj)

        # 2. Look for the point immediately to the left
        # Only do something if the neighbour is within the domain
        ii = ilist[jn]
        jj = jlist[jn] - 1

        if jj >= 0:
            # If the point is ocean, set it to '2'
            if inimask[ii, jj] == 1:
                inimask[ii, jj] = 2
                # Add the point to the list
                ilist.append(ii)
                jlist.append(jj)

        # 3. Look for the point immediately above
        # Only do something if the neighbour is within the domain
        ii = ilist[jn] + 1
        jj = jlist[jn]

        if ii < ni:
            # If the point is ocean, set it to '2'
            if inimask[ii, jj] == 1:
                inimask[ii, jj] = 2
                # Add the point to the list
                ilist.append(ii)
                jlist.append(jj)

        # 4. Look for the point immediately below
        # Only do something if the neighbour is within the domain
        ii = ilist[jn] - 1
        jj = jlist[jn]

        if ii >= 0:
            # If the point is ocean, set it to '2'
            if inimask[ii, jj] == 1:
                inimask[ii, jj] = 2
                # Add the point to the list
                ilist.append(ii)
                jlist.append(jj)

    # Control print the map
    plt.figure()
    plt.pcolor(JJ, II, inimask)
    plt.savefig("./figs/iteration" + str(k).zfill(4) + ".png")
    plt.close("all")

    # Remove the first n elements: indeed we know that those have already
    # been iterated from, so no need to do that again.

    del ilist[:n]
    del jlist[:n]
    # Update list length
    n = len(ilist)
    nm = np.sum(1 * (inimask == 2))  # number of points already marked '2'
    k += 1
    print(str(nm) + " points populated (+ " + str(1.0 * (nm - previous_nm) / previous_nm * 100) + "%)")

# Now, everything with '2' needs to go to '1' (the new mask)
# but before that, anything equal to 1 must go to 0 (the ocean
# that is not in the new mask)
maskout = inimask
maskout[inimask == 1] = 0
maskout[inimask == 2] = 1

if create_gif:
    import imageio
    with imageio.get_writer('figs/animated.gif', mode='I') as writer:
        for filename in os.listdir('figs'):
            image = imageio.imread(os.path.join('figs', filename))
            writer.append_data(image)
            os.remove(os.path.join('figs', filename))

# Save as NetCDF
#
fo = Dataset(outfile, 'w')
fo.createDimension('x', nj)
fo.createDimension('y', ni)
fo.createDimension('z', 1)
fo.createDimension('t', 1)

masko = fo.createVariable(varoutname, 'i4', ('t', 'y', 'x'))
masko[:] = maskout
fo.close()

print("File " + outfile + " created.")
